/*
 * Utility class to process read DNA strings
 */
#ifndef READPROCESSOR_H_
#define READPROCESSOR_H_

#include <map>
#include <vector>
#include <math.h>


#include <iostream>
using namespace std;


class ReadProcessor {

    int k;
    int n;

    std::map<char, char> nucleo_rev_cpl;
    unsigned long mask_kMer;

    void init_nucleo_rev_cpl() {
        nucleo_rev_cpl['A']='T';
        nucleo_rev_cpl['T']='A';
        nucleo_rev_cpl['C']='G';
        nucleo_rev_cpl['G']='C';
        nucleo_rev_cpl['N']='N';
    }

    void init_mask(int k){
        int nb_bits=2*k;
        mask_kMer=pow(2,nb_bits);
        mask_kMer=mask_kMer-1;
    }

    friend void testDNAToNumberSimple();

    unsigned long nucleoToNumber(char s) {
        unsigned long nbr;
        switch(s)
        {
            case 'A':
                nbr=0;
                break;
            case 'C':
                nbr=1;
                break;
            case 'G':
                nbr=2;
                break;
            case 'T':
                nbr=3;
                break;
            case 'N':
                nbr=0;
                break;
            default:
                cout<<"found unexpected character in DNA string: "<<s<<endl;
                throw -1;
        }
        return nbr;
    }

    unsigned long nucleoToNumberReverse(char s,int i) {
        unsigned long nbr=0;
        char cpl=nucleo_rev_cpl[s];
        nbr=nucleoToNumber(cpl);
        nbr=nbr<<2*(i-1);
        return nbr;
    }



public:
    ReadProcessor(int k=3) {
        n=0;
        set_k(k);
        init_nucleo_rev_cpl();
    }
    void set_k(int k) {
        this->k=k;
        init_mask(k);
    }

    // Do not inline this; it is slightly slower.    
    unsigned long kMerToNumberReverse(char * k_m,unsigned long * p_prev);


    unsigned long kMerToNumber(char * k_m,unsigned long * p_prev);


    unsigned long nucleoToNumberWrap(char s);

    void getKMerNumbers(char * dnaStr,int l,std::vector<unsigned long>& my_vect);


};

#endif
