#!/bin/sh
[ -z "$srcdir" ] && srcdir="."
echo "srcdir"=${srcdir}

# Gather here the tests that require at least 4 GB of memory to run.
## do real work
## Test high filter
echo " "
echo "##################################################################################"
echo "testing high filter"


../src/rock -C 100 -c 0 -l 2  -i ${srcdir}/data/iofiles.args/extract_klebsiella_long_reads_100.txt -o ${srcdir}/data/iofiles.args/extract_klebsiella_long_reads_100_filtered.txt >/dev/null || exit 101

# output files should be the same size, contain the same elements but not in the same order.
nb_PE1=`grep -c "@" ${srcdir}/data/fastq.raw/klebsiella_100_1.fq`
nb_PE2=`grep -c "@" ${srcdir}/data/fastq.raw/klebsiella_100_2.fq`


#ls -l ${srcdir}/
#ls -l ${srcdir}/data/
#ls -l data/fastq.filtered/

[ -f "data/fastq.filtered/SRR1222430_1.filtered.fastq" ] || exit 102
[ -f "data/fastq.filtered/SRR1222430_2.filtered.fastq" ] || exit 103

nb_PE1_filtered=`grep -c "@" data/fastq.filtered/SRR1222430_1.filtered.fastq`
nb_PE2_filtered=`grep -c "@" data/fastq.filtered/SRR1222430_2.filtered.fastq`

test $nb_PE1 -eq $nb_PE1_filtered || exit 104
test $nb_PE2 -eq $nb_PE2_filtered || exit 105

# test low filter.
echo " "
echo "##################################################################################"
echo "testing low filter"
../src/rock -C 100 -c 99 -l 2  -i ${srcdir}/data/iofiles.args/extract_klebsiella_long_reads_100.txt -o ${srcdir}/data/iofiles.args/extract_klebsiella_long_reads_100_filtered.txt >/dev/null || exit 106

[ -f "data/fastq.filtered/SRR1222430_1.filtered.fastq" ] || exit 107
[ -f "data/fastq.filtered/SRR1222430_2.filtered.fastq" ] || exit 108

nb_PE1_filtered=`grep -c "@" data/fastq.filtered/SRR1222430_1.filtered.fastq`
nb_PE2_filtered=`grep -c "@" data/fastq.filtered/SRR1222430_2.filtered.fastq`

test $nb_PE1_filtered -eq 0 || exit 109
test $nb_PE2_filtered -eq 0 || exit 110

# test that input fastq file names can be provided in command-line.
echo " "
echo "##################################################################################"
echo "testing that input fastq file names can be provided in command line."
../src/rock -C 100 -c 1 -l 2  ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq ${srcdir}/data/unit/test_single.fq ${srcdir}/data/unit/test_single2.fq >/dev/null || exit 111

[ -f "klebsiella_100_1.rock.fq" ] || exit 112
[ -f "klebsiella_100_2.rock.fq" ] || exit 113
[ -f "test_single.rock.fq" ] || exit 114
[ -f "test_single2.rock.fq" ] || exit 115



# checking that output files were sorted in decreasing order of quality score. For that expect to have SRR122430.1.1 as 1rst record in filtered file.
ret=`head -4 "klebsiella_100_1.rock.fq"|grep -c "SRR1222430.1.1"`
test $ret -eq 2 || exit 116

echo "erasing test result files"
rm -f "klebsiella_100_1.rock.fq" || exit 117
rm -f "klebsiella_100_2.rock.fq" || exit 118
rm -f "test_single.rock.fq"|| exit 119
rm -f "test_single2.rock.fq"|| exit 120
rm -f "klebsiella_100_1.undefined.fq" || exit 121
rm -f "klebsiella_100_2.undefined.fq" || exit 122
rm -f "test_single.undefined.fq" || exit 123
rm -f "test_single2.undefined.fq" || exit 124



# test the -v option
echo " "
echo "##################################################################################"
echo "testing verbose mode"
../src/rock -C 100 -c 1 -l 2  -v ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq ${srcdir}/data/unit/test_single.fq ${srcdir}/data/unit/test_single2.fq|grep "CMS size=" >/dev/null || exit 125

../src/rock -C 100 -c 1 -v  -n 1000 ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq ${srcdir}/data/unit/test_single.fq ${srcdir}/data/unit/test_single2.fq|grep "expected probability of collision was:" >/dev/null || exit 126

echo "erasing test result files"
rm -f "klebsiella_100_1.rock.fq" || exit 127
rm -f "klebsiella_100_2.rock.fq" || exit 128
rm -f "test_single.rock.fq"|| exit 129
rm -f "test_single2.rock.fq"|| exit 130
rm -f "klebsiella_100_1.undefined.fq" || exit 131
rm -f "klebsiella_100_2.undefined.fq" || exit 132
rm -f "test_single.undefined.fq" || exit 133
rm -f "test_single2.undefined.fq" || exit 134

echo " "
echo "##################################################################################"
echo "testing rock with a quality score threshold for nucleotides."

expected_diff1="304a305,308 \
  > @SRR1222430.37 37 length=250 \
> GCCTTTTCTTTTTCCAGGGAAAACCATCCAGGAGGAACTTTATTATGGCGATGTATGAAGTCGGTACCGTCACGGGTGCCGCGTCGCAGGCACGGGTGACAGGTGCGACAACAAAATGGTCACAGGAGGCGCTGGGGATACAGCCCGGGTCGATTCTGGTGGTCTACCGCAGCGGTAGTGCTGACCTGTATGCGATCAAATCCGTGGACAGCGACACGCAACTGACGCTGACCCGGAATATCACCACCGC \
> +SRR1222430.37 37 length=250 \
> AAAABFFFFFFFGCGFGGEFFCHHEEFHHGHHAFEAGAGGGFGGFHGFHGEEEFF5BDHHBDEGCEEHGAEGGFFGFCFHCGGC@E/EEEEHFEC@BBFFFG/GFEFDDCDHHGCC<?FGDDGHFFFGFFCGGG-.AGDGBGGHHGGG?ACGCFGG99B9;?C99B0CDGGCFF;DDFFFFFFFB99:;BBF/A.:9DFFF/F?EEFFFFFF>ADAFDFFFFFFE/ADADDFFFFFA;EEFF/BFFEEFF"

expected_diff2="> @SRR1222430.37 37 length=251 \
> GCTGGCCAGCTGGTTAGCAAACGACGAGGTGCTGGCGGGTTTAGCGGGAAAAATTGCGGAAGAGGCGCCGGGAAAAGCGGGGGTGTTATTACAGGTCTACGGCAGATGCGTGTCGCTGGCCGCGGCTTTTAGCGCCTACAAGTCTGGACTTCCCCGTCGGGGGGCAACCACAATTCACCCCGGCTGTATTCCACTCGGCCCCGGTGACCCTTTTGGTGTCGCCCCCGGACACCGTGCCCGTGCCCCGGTAC \
> +SRR1222430.37 37 length=251 \
> A3>333BFA2FF4GBFFDGGCGED?FGEGGHGFFFEEG?AF13@50>///11B13@@1/>//B0?>////<//B/00??@/--:--.;:C000;:0/0009.-9:.00:-.;9/9...-;.--9@--9:////-9-9..////9/;//;9///.9-..--------..99.9.//////;-;--9-.////://9/9.;.-;-99-.//.;////-;?9;...9-9-----9;-.;.;/.-9.;/99=--;"

mkdir tmp
../src/rock -C 100 -k 30 -c 1 -l 2  -o ${srcdir}/data/iofiles.args/output_files_noNQ_Thres.txt  ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq >/dev/null || exit 140

../src/rock -C 100 -k 30 -c 1 -l 2 -q 2 -o ${srcdir}/data/iofiles.args/output_files_NQ_Thres_very_low.txt  ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq >/dev/null || exit 141

ret1=`diff tmp/klebsiella_100_1_very_low_qual_thres.fq tmp/klebsiella_100_1_no_qual_thres.fq|wc -l`
test $ret1 -eq 0 || exit 142
ret2=`diff tmp/klebsiella_100_2_very_low_qual_thres.fq tmp/klebsiella_100_2_no_qual_thres.fq|wc -l`
test $ret2 -eq 0 || exit 143

../src/rock -C 100 -k 30 -c 1 -l 2 -q 12 -o ${srcdir}/data/iofiles.args/output_files_NQ_Thres_12.txt ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq >/dev/null ||exit 144

../src/rock -C 100 -k 30 -c 1 -l 2 -q 13 -o ${srcdir}/data/iofiles.args/output_files_NQ_Thres_13.txt ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq >/dev/null ||exit 145

ret1=`diff tmp/klebsiella_100_1_13_qual_thres.fq tmp/klebsiella_100_1_12_qual_thres.fq|grep -c "@SRR1222430.37"`
test $ret1 -eq  1 || exit 146

ret2=`diff tmp/klebsiella_100_1_13_qual_thres.fq tmp/klebsiella_100_1_12_qual_thres.fq|grep -c "length"`
test $ret2 -eq  2 || exit 147

ret1=`diff tmp/klebsiella_100_2_13_qual_thres.fq tmp/klebsiella_100_2_12_qual_thres.fq|grep -c "@SRR1222430.37"`
test $ret1 -eq  1 || exit 148

ret2=`diff tmp/klebsiella_100_2_13_qual_thres.fq tmp/klebsiella_100_2_12_qual_thres.fq|grep -c "length"`
test $ret2 -eq  2 || exit 149

rm -fr tmp

echo " "
echo "##################################################################################"
echo "testing ROCK with a quality score threshold for nucleotides and minimum number of valid k-mer to keep a read."

mkdir tmp
../src/rock -C 100 -k 30 -c 1 -l 2 -o ${srcdir}/data/iofiles.args/output_files_noNQ_Thres.txt  ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq >/dev/null || exit 150

../src/rock -C 100 -k 30 -c 1 -l 2 -q 2 -m 5 -o ${srcdir}/data/iofiles.args/output_files_NQ_Thres_very_low.txt  ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq >/dev/null || exit 151

ret1=`diff tmp/klebsiella_100_1_very_low_qual_thres.fq tmp/klebsiella_100_1_no_qual_thres.fq|wc -l`
test $ret1 -eq 0 || exit 152
ret2=`diff tmp/klebsiella_100_2_very_low_qual_thres.fq tmp/klebsiella_100_2_no_qual_thres.fq|wc -l`
test $ret2 -eq 0 || exit 153


[ -f "tmp/klebsiella_100_1.undefined.fq" ] || exit 154
[ -f "tmp/klebsiella_100_2.undefined.fq" ] || exit 155

ret1=`cat tmp/klebsiella_100_1.undefined.fq|wc -l`
ret2=`cat tmp/klebsiella_100_2.undefined.fq|wc -l`

test $ret1 -eq 0 || exit 156
test $ret1 -eq 0 || exit 157

../src/rock -k 30 -C 100 -c 1 -l 2 -q 13 -m 500 -o ${srcdir}/data/iofiles.args/output_files_NQ_Thres_13.txt ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq >/dev/null || exit 158

[ -f "tmp/klebsiella_100_1.undefined.fq" ] || exit 159
[ -f "tmp/klebsiella_100_2.undefined.fq" ] || exit 160

ret1=`cat tmp/klebsiella_100_1.undefined.fq|wc -l`
ret2=`cat tmp/klebsiella_100_2.undefined.fq|wc -l`

test $ret1 -eq 400 || exit 161
test $ret1 -eq 400 || exit 162

../src/rock -k 30 -C 100 -c 1 -l 2 -q 13 -m 300 -o ${srcdir}/data/iofiles.args/output_files_NQ_Thres_13.txt ${srcdir}/data/fastq.raw/klebsiella_100_1.fq,${srcdir}/data/fastq.raw/klebsiella_100_2.fq >/dev/null || exit 163

[ -f "tmp/klebsiella_100_1.undefined.fq" ] || exit 164
[ -f "tmp/klebsiella_100_2.undefined.fq" ] || exit 165

ret1=`cat tmp/klebsiella_100_1.undefined.fq|wc -l`
ret2=`cat tmp/klebsiella_100_2.undefined.fq|wc -l`

test $ret1 -eq 136 || exit 166
test $ret1 -eq 136 || exit 167

echo " "
echo "***##################################################################################"
#echo "testing ROCK with a quality score threshold for nucleotides and PE processed separately with strict filter "
#../src/rock -C 100 -c 1 -l 2 -q 2 -p 1 -o ${srcdir}/data/iofiles.args/output_files_2_PE_separated.txt ${srcdir}/data/fastq.raw/klebsiella_5_1_bad_scores.fq,${srcdir}/data/fastq.raw/klebsiella_5_2_bad_scores.fq >/dev/null || exit 168

#[ -f "tmp/klebsiella_5_1_bad_scores.undefined.fq" ] || exit 169
#[ -f "tmp/klebsiella_5_2_bad_scores.undefined.fq" ] || exit 170

# ret1=`cat tmp/klebsiella_5_1_bad_scores.undefined.fq|wc -l`
# ret2=`cat tmp/klebsiella_5_2_bad_scores.undefined.fq|wc -l`
# 
# test $ret1 -eq 8 || exit 171
# test $ret2 -eq 8 || exit 171
# 
# ret1=`grep -c SRR1222430.1 tmp/klebsiella_5_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.1 tmp/klebsiella_5_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 2 || exit 172
# test $ret2 -eq 2 || exit 172
# 
# ret1=`grep -c SRR1222430.4 tmp/klebsiella_5_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.4 tmp/klebsiella_5_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 2 || exit 173
# test $ret2 -eq 2 || exit 173
# 
# ret1=`grep -c SRR1222430.2 tmp/klebsiella_5_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.2 tmp/klebsiella_5_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 0 || exit 174
# test $ret2 -eq 0 || exit 174
# 
# ret1=`grep -c SRR1222430.3 tmp/klebsiella_5_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.3 tmp/klebsiella_5_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 0 || exit 175
# test $ret2 -eq 0 || exit 175
# 
# ret1=`grep -c SRR1222430.5 tmp/klebsiella_5_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.5 tmp/klebsiella_5_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 0 || exit 176
# test $ret2 -eq 0 || exit 176
# 
# ret1=`cat tmp/klebsiella_5_1_2_qual_thres.fq|wc -l`
# ret2=`cat tmp/klebsiella_5_1_2_qual_thres.fq|wc -l`
# 
# test $ret1 -eq 8 || exit 177
# test $ret2 -eq 8 || exit 177
# 
# ret1=`grep -c SRR1222430.2 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.2 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 178
# test $ret2 -eq 2 || exit 178
# 
# ret1=`grep -c SRR1222430.3 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.3 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 0 || exit 179
# test $ret2 -eq 0 || exit 179
# 
# ret1=`grep -c SRR1222430.5 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.5 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 180
# test $ret2 -eq 2 || exit 180
# 
# rm -f tmp/klebsiella_5_1_bad_scores.undefined.fq
# rm -f tmp/klebsiella_5_2_bad_scores.undefined.fq
# rm -f tmp/klebsiella_5_1_2_qual_thres.fq
# rm -f tmp/klebsiella_5_2_2_qual_thres.fq


# echo " "
# echo "##################################################################################"
# echo "testing ROCK with a quality score threshold for nucleotides, PE processed separately (strict filter)  and -m option"
# ../src/rock -C 100 -c 1 -l 2 -q 2 -m 2 -p 1 -o ${srcdir}/data/iofiles.args/output_files_2_PE_separated.txt ${srcdir}/data/fastq.raw/klebsiella_5_1_bad_scores.fq,${srcdir}/data/fastq.raw/klebsiella_5_2_bad_scores.fq >/dev/null || exit 281
# 
# [ -f "tmp/klebsiella_5_1_bad_scores.undefined.fq" ] || exit 182
# [ -f "tmp/klebsiella_5_2_bad_scores.undefined.fq" ] || exit 182
# 
# ret1=`cat tmp/klebsiella_5_1_bad_scores.undefined.fq|wc -l`
# ret2=`cat tmp/klebsiella_5_2_bad_scores.undefined.fq|wc -l`
# 
# test $ret1 -eq 8 || exit 183
# test $ret2 -eq 8 || exit 183
# 
# ret1=`grep -c SRR1222430.1 tmp/klebsiella_5_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.1 tmp/klebsiella_5_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 2 || exit 184
# test $ret2 -eq 2 || exit 184
# 
# ret1=`grep -c SRR1222430.4 tmp/klebsiella_5_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.4 tmp/klebsiella_5_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 2 || exit 185
# test $ret2 -eq 2 || exit 185
# 
# ret1=`grep -c SRR1222430.2 tmp/klebsiella_5_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.2 tmp/klebsiella_5_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 0 || exit 186
# test $ret2 -eq 0 || exit 186
# 
# ret1=`grep -c SRR1222430.3 tmp/klebsiella_5_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.3 tmp/klebsiella_5_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 0 || exit 187
# test $ret2 -eq 0 || exit 187
# 
# ret1=`grep -c SRR1222430.5 tmp/klebsiella_5_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.5 tmp/klebsiella_5_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 0 || exit 188
# test $ret2 -eq 0 || exit 188
# 
# ret1=`cat tmp/klebsiella_5_1_2_qual_thres.fq|wc -l`
# ret2=`cat tmp/klebsiella_5_1_2_qual_thres.fq|wc -l`
# 
# test $ret1 -eq 8 || exit 189
# test $ret2 -eq 8 || exit 189
# 
# ret1=`grep -c SRR1222430.2 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.2 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 190
# test $ret2 -eq 2 || exit 190
# 
# ret1=`grep -c SRR1222430.3 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.3 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 0 || exit 191
# test $ret2 -eq 0 || exit 191
# 
# ret1=`grep -c SRR1222430.5 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.5 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 192
# test $ret2 -eq 2 || exit 192
# 
# rm -f tmp/klebsiella_5_1_bad_scores.undefined.fq
# rm -f tmp/klebsiella_5_2_bad_scores.undefined.fq
# rm -f tmp/klebsiella_5_1_2_qual_thres.fq
# rm -f tmp/klebsiella_5_2_2_qual_thres.fq
# 


# echo " "
# echo "##################################################################################"
# echo "testing ROCK with no quality score threshold for nucleotides, no low filter and PE processed separately (strict filter)"
# 
# ../src/rock -C 100 -c 0 -l 2 -q 0 -p 1 -o ${srcdir}/data/iofiles.args/output_files_2_PE_separated.txt ${srcdir}/data/fastq.raw/klebsiella_5_1_bad_scores.fq,${srcdir}/data/fastq.raw/klebsiella_5_2_bad_scores.fq >/dev/null || exit 193
# 
# [ -f "tmp/klebsiella_5_1_bad_scores.undefined.fq" ] || exit 194
# [ -f "tmp/klebsiella_5_2_bad_scores.undefined.fq" ] || exit 194
# 
# ret1=`cat tmp/klebsiella_5_1_bad_scores.undefined.fq|wc -l`
# ret2=`cat tmp/klebsiella_5_2_bad_scores.undefined.fq|wc -l`
# 
# test $ret1 -eq 0 || exit 195
# test $ret2 -eq 0 || exit 195
# 
# ret1=`cat tmp/klebsiella_5_1_2_qual_thres.fq|wc -l`
# ret2=`cat tmp/klebsiella_5_1_2_qual_thres.fq|wc -l`
# 
# test $ret1 -eq 20 || exit 196
# test $ret2 -eq 20 || exit 196
# 
# ret1=`grep -c SRR1222430.2 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.2 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 197
# test $ret2 -eq 2 || exit 197
# 
# ret1=`grep -c SRR1222430.3 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.3 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 198
# test $ret2 -eq 2 || exit 198
# 
# ret1=`grep -c SRR1222430.4 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.4 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 199
# test $ret2 -eq 2 || exit 199
# 
# ret1=`grep -c SRR1222430.1 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.1 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 200
# test $ret2 -eq 2 || exit 200
# 
# ret1=`grep -c SRR1222430.5 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.5 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 201
# test $ret2 -eq 2 || exit 201
# 
# rm -f tmp/klebsiella_5_1_bad_scores.undefined.fq
# rm -f tmp/klebsiella_5_2_bad_scores.undefined.fq
# rm -f tmp/klebsiella_5_1_2_qual_thres.fq
# rm -f tmp/klebsiella_5_2_2_qual_thres.fq


# echo " "
# echo "##################################################################################"
# echo "testing ROCK with no quality score threshold for nucleotides, no low filter and PE processed separately (lenient filter). Results should be the same as with strict filter"
# 
# ../src/rock -C 100 -c 0 -l 2 -q 0 -p 2 -o ${srcdir}/data/iofiles.args/output_files_2_PE_separated.txt ${srcdir}/data/fastq.raw/klebsiella_5_1_bad_scores.fq,${srcdir}/data/fastq.raw/klebsiella_5_2_bad_scores.fq >/dev/null || exit 202
# 
# [ -f "tmp/klebsiella_5_1_bad_scores.undefined.fq" ] || exit 202
# [ -f "tmp/klebsiella_5_2_bad_scores.undefined.fq" ] || exit 202
# 
# ret1=`cat tmp/klebsiella_5_1_bad_scores.undefined.fq|wc -l`
# ret2=`cat tmp/klebsiella_5_2_bad_scores.undefined.fq|wc -l`
# 
# test $ret1 -eq 0 || exit 203
# test $ret2 -eq 0 || exit 203
# 
# ret1=`cat tmp/klebsiella_5_1_2_qual_thres.fq|wc -l`
# ret2=`cat tmp/klebsiella_5_1_2_qual_thres.fq|wc -l`
# 
# test $ret1 -eq 20 || exit 204
# test $ret2 -eq 20 || exit 204
# 
# ret1=`grep -c SRR1222430.2 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.2 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 205
# test $ret2 -eq 2 || exit 205
# 
# ret1=`grep -c SRR1222430.3 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.3 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 206
# test $ret2 -eq 2 || exit 206
# 
# ret1=`grep -c SRR1222430.4 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.4 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 207
# test $ret2 -eq 2 || exit 207
# 
# ret1=`grep -c SRR1222430.1 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.1 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 208
# test $ret2 -eq 2 || exit 208
# 
# ret1=`grep -c SRR1222430.5 tmp/klebsiella_5_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.5 tmp/klebsiella_5_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 209
# test $ret2 -eq 2 || exit 209
# 
# 
# rm -f tmp/klebsiella_5_1_bad_scores.undefined.fq
# rm -f tmp/klebsiella_5_2_bad_scores.undefined.fq
# rm -f tmp/klebsiella_5_1_2_qual_thres.fq
# rm -f tmp/klebsiella_5_2_2_qual_thres.fq


# echo " "
# echo "##################################################################################"
# echo "testing ROCK with a quality score threshold for nucleotides, PE processed separately (lenient filter)  and -m option"
# ../src/rock -C 100 -c 1 -l 2 -q 2 -m 2 -p 2 -o ${srcdir}/data/iofiles.args/output_files_2_PE_separated_lenient.txt ${srcdir}/data/fastq.raw/klebsiella_6_1_bad_scores.fq,${srcdir}/data/fastq.raw/klebsiella_6_2_bad_scores.fq >/dev/null || exit 210
# 
# [ -f "tmp/klebsiella_6_1_bad_scores.undefined.fq" ] || exit 211
# [ -f "tmp/klebsiella_6_2_bad_scores.undefined.fq" ] || exit 211
# 
# ret1=`cat tmp/klebsiella_6_1_bad_scores.undefined.fq|wc -l`
# ret2=`cat tmp/klebsiella_6_2_bad_scores.undefined.fq|wc -l`
# 
# test $ret1 -eq 8 || exit 212
# test $ret2 -eq 8 || exit 212
# 
# ret1=`grep -c SRR1222430.1 tmp/klebsiella_6_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.1 tmp/klebsiella_6_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 2 || exit 213
# test $ret2 -eq 2 || exit 213
# 
# ret1=`grep -c SRR1222430.4 tmp/klebsiella_6_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.4 tmp/klebsiella_6_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 2 || exit 214
# test $ret2 -eq 2 || exit 214
# 
# ret1=`grep -c SRR1222430.2 tmp/klebsiella_6_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.2 tmp/klebsiella_6_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 0 || exit 215
# test $ret2 -eq 0 || exit 215
# 
# ret1=`grep -c SRR1222430.3 tmp/klebsiella_6_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.3 tmp/klebsiella_6_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 0 || exit 216
# test $ret2 -eq 0 || exit 216
# 
# ret1=`grep -c SRR1222430.5 tmp/klebsiella_6_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.5 tmp/klebsiella_6_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 0 || exit 217
# test $ret2 -eq 0 || exit 217
# 
# ret1=`grep -c SRR1222430.6 tmp/klebsiella_6_1_bad_scores.undefined.fq`
# ret2=`grep -c SRR1222430.6 tmp/klebsiella_6_2_bad_scores.undefined.fq`
# 
# test $ret1 -eq 0 || exit 218
# test $ret2 -eq 0 || exit 218
# 
# ret1=`cat tmp/klebsiella_6_1_2_qual_thres.fq|wc -l`
# ret2=`cat tmp/klebsiella_6_1_2_qual_thres.fq|wc -l`
# 
# test $ret1 -eq 8 || exit 219
# test $ret2 -eq 8 || exit 219
# 
# ret1=`grep -c SRR1222430.2 tmp/klebsiella_6_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.2 tmp/klebsiella_6_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 220
# test $ret2 -eq 2 || exit 220
# 
# ret1=`grep -c SRR1222430.3 tmp/klebsiella_6_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.3 tmp/klebsiella_6_2_2_qual_thres.fq`
# 
# test $ret1 -eq 0 || exit 221
# test $ret2 -eq 0 || exit 221
# 
# ret1=`grep -c SRR1222430.5 tmp/klebsiella_6_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.5 tmp/klebsiella_6_2_2_qual_thres.fq`
# 
# test $ret1 -eq 2 || exit 222
# test $ret2 -eq 2 || exit 222
# 
# ret1=`grep -c SRR1222430.6 tmp/klebsiella_6_1_2_qual_thres.fq`
# ret2=`grep -c SRR1222430.6 tmp/klebsiella_6_2_2_qual_thres.fq`
# 
# test $ret1 -eq 0 || exit 223
# test $ret2 -eq 0 || exit 223
# 
# mv tmp/klebsiella_6_1_bad_scores.undefined.fq tmp/klebsiella_6_1_bad_scores.undefined.fq.lenient
# mv tmp/klebsiella_6_2_bad_scores.undefined.fq tmp/klebsiella_6_2_bad_scores.undefined.fq.lenient
# mv tmp/klebsiella_6_1_2_qual_thres.fq tmp/klebsiella_6_1_2_qual_thres.fq.lenient
# mv tmp/klebsiella_6_2_2_qual_thres.fq tmp/klebsiella_6_2_2_qual_thres.fq.lenient
# 
# echo " Now use the scrict filter on the same data and check that the only difference is read SRR1222430.6 that should be kept by scrict filter"
# 
# ../src/rock -C 100 -c 1 -l 2 -q 2 -m 2 -p 1 -o ${srcdir}/data/iofiles.args/output_files_2_PE_separated_lenient.txt ${srcdir}/data/fastq.raw/klebsiella_6_1_bad_scores.fq,${srcdir}/data/fastq.raw/klebsiella_6_2_bad_scores.fq >/dev/null || exit 224
# 
# ret1=`diff tmp/klebsiella_6_1_bad_scores.undefined.fq tmp/klebsiella_6_1_bad_scores.undefined.fq.lenient|wc -l`
# test $ret1 -eq 0 ||exit 225
# 
# ret2=`diff tmp/klebsiella_6_2_bad_scores.undefined.fq tmp/klebsiella_6_2_bad_scores.undefined.fq.lenient|wc -l`
# test $ret2 -eq 0 ||exit 226
# 
# ret3=`diff tmp/klebsiella_6_1_2_qual_thres.fq tmp/klebsiella_6_1_2_qual_thres.fq.lenient|grep -c SRR1222430.6`
# test $ret3 -eq 2 ||exit 227
# 
# ret4=`diff tmp/klebsiella_6_1_2_qual_thres.fq tmp/klebsiella_6_1_2_qual_thres.fq.lenient|wc -l`
# test $ret4 -eq 5 || exit 228 # put 5 because there are 4 lines for 1 read and 1rts line that looks like: "9,12d8"
# 
# ret5=`diff tmp/klebsiella_6_2_2_qual_thres.fq tmp/klebsiella_6_2_2_qual_thres.fq.lenient|grep -c SRR1222430.6`
# test $ret5 -eq 2 ||exit 229
# 
# ret6=`diff tmp/klebsiella_6_2_2_qual_thres.fq tmp/klebsiella_6_2_2_qual_thres.fq.lenient|wc -l`
# test $ret6 -eq 5 || exit 230
# 
# 
# 
# rm -f tmp/klebsiella_6_1_bad_scores.undefined.fq
# rm -f tmp/klebsiella_6_2_bad_scores.undefined.fq
# rm -f tmp/klebsiella_6_1_2_qual_thres.fq
# rm -f tmp/klebsiella_6_2_2_qual_thres.fq
# rm -fr tmp/klebsiella_6_1_bad_scores.undefined.fq.lenient
# rm -fr tmp/klebsiella_6_2_bad_scores.undefined.fq.lenient
# rm -fr tmp/klebsiella_6_1_2_qual_thres.fq.lenient
# rm -fr tmp/klebsiella_6_2_2_qual_thres.fq.lenient



rm -fr tmp


echo " "
echo "##################################################################################"
echo "unit testing for cms component"
../src/unit_test_cms || exit 240

exit 0

     
     
